#ifndef GIGASET_H
#define GIGASET_H



#include <termios.h>
#include <unistd.h>



#define GIGA_NUM_COLUMNS 80
#define GIGA_NUM_LINES   25

#define GIGA_OK 0
#define GIGA_ERROR -1
#define GIGA_FAILURE -2
#define GIGA_BASE_FULL -3
#define GIGA_INVALID_PIN -4
#define GIGA_LINES_EXCEED -5
#define GIGA_COLUMNS_EXCEED -6
#define GIGA_ARG_MODE_ERROR -100
#define GIGA_ARG_RATE_ERROR -101
#define GIGA_ARG_FORMAT_ERROR -102
#define GIGA_ARG_HANDSHAKE_ERROR -103
#define GIGA_ARG_NAME_ERROR -104
#define GIGA_ARG_TYPE_ERROR -105
#define GIGA_ARG_PIN_ERROR -106



class Gigaset {

public:

  /* basic parameters */
  int mode;
  int rate;
  int format;
  int handshake;
  int type;
  int connected;
  int strength;
  int quality;

  /* information about local station */
  char local_hardware[GIGA_NUM_LINES];
  char local_firmware[GIGA_NUM_LINES];
  char local_id[GIGA_NUM_LINES];
  char local_name[GIGA_NUM_LINES];

  /* information about peer station */
  char partner_hardware[GIGA_NUM_LINES];
  char partner_firmware[GIGA_NUM_LINES];
  char partner_id[GIGA_NUM_LINES];
  char partner_name[GIGA_NUM_LINES];

  /* basic input functions */
  int readParameter();
  int readStationInfo();

  /* basic output functions */
  int writeParameter();
  int writeName(char* name,int index=0);
  int writeType();

  /* operations */
  void initSequence();
  int startRegistration();
  int registerStation(char* pin);
  int changePIN(char* oldpin,char* newpin);
  int reset();
  int operate();

  /* gigaset communication */
  int communicate(char* cmd,int nowait=0);
  int timeout;
  char buffer[GIGA_NUM_LINES][GIGA_NUM_COLUMNS];
  int z,s;

  /* basics */
  Gigaset();
  ~Gigaset();
  openDevice(char* dev);
  closeDevice();

  int fd;
  char* device;
  struct termios tio_new;
  struct termios tio_old;

  int showstat;

};



#endif







