#include <stdio.h>
#include <stdlib.h>

#include "gigaset.h"


void operate(Gigaset& giga) {
  if(giga.operate()<GIGA_OK) {
    printf("Error while switching to operation-mode\n");      
    exit(-1);
  };
};


int main(int argc,char* argv[]) {

  if(argc < 2) {
    printf("\nGigaset M101 Data configuration tool\n");
    printf("usage: gigactrl <device> [[option][parameter]]*\n");
    printf("options/parameter: (factor defaults are marked with '*'\n");
    printf("\n-m <mode> ... set serial operating mode\n");
    printf("    0 ___ direct connection\n");
    printf("    1 ___ AT-mode PC\n");
    printf("    2 *__ AT-mode modem\n");
    printf("    3 ___ AVM/Eumex-mode PC\n");
    printf("    4 ___ AVM/Eumex-mode modem\n");
    printf("\n-r <baudrate> ... set serial baudrate\n");
    printf("    0 ______ 300 Bit/sec.\n");
    printf("    1 ______ 600 Bit/sec.\n");
    printf("    2 _____ 1200 Bit/sec.\n");
    printf("    3 _____ 2400 Bit/sec.\n");
    printf("    4 _____ 4800 Bit/sec.\n");
    printf("    5 _____ 9600 Bit/sec.\n");
    printf("    6 ____ 19200 Bit/sec.\n");
    printf("    7 *___ 38400 Bit/sec.\n");
    printf("    8 ____ 57600 Bit/sec.\n");
    printf("    9 ___ 115200 Bit/sec.\n");
    printf("\n-f <format> ... set serial format\n");
    printf("    0 ___ 7E1\n");
    printf("    1 ___ 7O1\n");
    printf("    2 *__ 8N1\n");
    printf("\n-h <handshake> ... set serial handshake\n");
    printf("    0 ___ none\n");
    printf("    1 ___ XON/XOFF\n");
    printf("    2 *__ RTS/CTS\n");
    printf("    3 ___ DTR/DSR\n");
    printf("    4 ___ XON/XOFF uni PC->GM100\n");
    printf("    5 ___ XON/XOFF uni GM100->DTE\n");
    printf("    6 ___ XON/XOFF & RTS/CTS\n");
    printf("\n-ln <name> ... change name of local station\n");
    printf("    string ___ 20 characters (no commas allowed)\n");
    printf("\n-pn <name> ... change name of partner station\n");
    printf("    string ___ 20 characters (no commas allowed)\n");
    printf("\n--listen ... listen for registration (base only)\n");
    printf("\n--register <pin> ... register to base (subscriber only)\n");
    printf("    number ___ up to five digits\n");
    printf("\n--settype <type> ... change type of local station\n");
    printf("    0 *__ base-station\n");
    printf("    1 ___ subscriber-station\n");
    printf("\n--changepin <old> <new> ... change pin of base station\n");
    printf("    number number ___ up to five digits\n");
    printf("\n--reset ... reset Gigaset (factory defaults)\n");
    printf("\n--terminal ... at-command level\n");
    printf("\noptions with leading '--' have to be used stand alone\n");
    exit(0);
  };

  Gigaset giga;
  giga.showstat=1;
  char* device=argv[1];
  
  if(giga.openDevice(device) < GIGA_OK) {
    printf("Error while opening device %s\n",device);
    exit(-1);
  };
  
  if(argc==2) {
     sleep(1);

     if(giga.readStationInfo()<GIGA_OK) {
       printf("Error while reading data\n");
       exit(-1);
     };
     
     printf("\nGeneral Information\n");
     printf("Mode ....... %i\n",giga.mode);
     
     if(giga.mode==0) {
       
       printf("Rate ....... %i\n",giga.rate);
       printf("Format ..... %i\n",giga.format);
       printf("Handshake .. %i\n",giga.handshake);
       
     };
     
     printf("Strength ... %i\n",giga.strength);
     printf("Quality .... %i\n",giga.quality);
     
     printf("\nLocal Station: %i\n",giga.type);
     printf("Hardware ... %s\n",giga.local_hardware);
     printf("Firmware ... %s\n",giga.local_firmware);
     printf("Modem-ID ... %s\n",giga.local_id);
     printf("Name ....... %s\n",giga.local_name);
     
     if(giga.connected) {
       
       printf("\nPartner Station: connected\n");
       printf("Hardware ... %s\n",giga.partner_hardware);
       printf("Firmware ... %s\n",giga.partner_firmware);
       printf("Modem-ID ... %s\n",giga.partner_id);
       printf("Name ....... %s\n",giga.partner_name);
       
     }
     else {
       printf("\nPartner Station: disconnected\n");
     };
     
     printf("\n");

     operate(giga);
     
     exit(0);
  };

  if(strcmp(argv[2],"--reset")==0) {
    if(giga.reset()<GIGA_OK) {
      printf("Error while reseting local station\n");
      exit(-1);
    };
    exit(0);
  };

  if(strcmp(argv[2],"--settype")==0) {
    if(argc!=4) {
      printf("Error: too many or missing arguments\n");
      exit(-1);      
    };
    giga.type=atoi(argv[3]);
    if(giga.writeType()) {
      printf("Error while changing type\n");
      exit(-1);
    };
    exit(0);
  };  

  giga.readStationInfo();

  if(strcmp(argv[2],"--listen")==0) {
    if(giga.type!=0) {
      printf("Error: local station is no basis\n");
      exit(-1);
    };
    if(giga.startRegistration()<GIGA_OK) {
      printf("Error while waiting for partner\n");
      exit(-1);
    };
    operate(giga);
    exit(0);
  };

  if(strcmp(argv[2],"--register")==0) {
    if(argc!=4) {
      printf("Error: too many or missing arguments\n");
      exit(-1);      
    };
    char* pin=argv[3];
    if(giga.type!=1) {
      printf("Error: can't register - local station is basis\n");
      exit(-1);
    };
    if(giga.registerStation(pin)<GIGA_OK) {
      printf("Error while waiting for partner\n");
      exit(-1);
    };
    operate(giga);
    exit(0);
  };

  if(strcmp(argv[2],"--changepin")==0) {
    if(argc!=5) {
      printf("Error: too many or missing arguments\n");
      exit(-1);      
    };
    char* oldpin=argv[3];
    char* newpin=argv[4];
    if(giga.changePIN(oldpin,newpin)<GIGA_OK) {
      printf("Error while changing pin\n");
      exit(-1);
    };
    operate(giga);
    exit(0);
  };

  if(strcmp(argv[2],"--terminal")==0) {
    while(1) {

      char buf[1024];
      printf("AT+SDV24");
      scanf("%s",buf);

      if(strcmp(buf,".")==0) break;

      giga.communicate(buf);
      for(int i=1;i<=giga.z;i++) {
	printf("%s\n",giga.buffer[i]);
      };
    };
    operate(giga);
    exit(0);
  };

  int arg=2;
  int parameterflag=0;

  while(arg<argc) {

    if((argc-arg)<2) {
      printf("Error: argument missing\n");
      exit(-1);      
    };

    if(strcmp(argv[arg],"-m")==0) {
      giga.mode=atoi(argv[(++arg)++]);
      parameterflag=1;
      continue;
    };

    if(strcmp(argv[arg],"-r")==0) {
      giga.rate=atoi(argv[(++arg)++]);
      parameterflag=1;
      continue;
    };  
    
    if(strcmp(argv[arg],"-f")==0) {
      giga.format=atoi(argv[(++arg)++]);
      parameterflag=1;
      continue;
    };  
    
    if(strcmp(argv[arg],"-h")==0) {
      giga.handshake=atoi(argv[(++arg)++]);
      parameterflag=1;
      continue;
    };  
    
    if(strcmp(argv[arg],"-ln")==0) {
      if(giga.writeName(argv[(++arg)++],0)<GIGA_OK) {
	printf("Error: while changing name\n");
	exit(-1);
      };
      continue;
    };

    if(strcmp(argv[arg],"-pn")==0) {
      if(giga.writeName(argv[(++arg)++],1)<GIGA_OK) {
	printf("Error: while changing name\n");
	exit(-1);
      };
      continue;
    };

    printf("Error: unknown option %s\n",argv[arg++]);

  };

  if(parameterflag==1) {
    if(giga.writeParameter()<GIGA_OK) {
       printf("Error: while writing serial parameter\n");     
       exit(-1);
    };
  };
  
  operate(giga);
  giga.closeDevice();

};





